from .TimerDTO import Timer
from datetime import datetime


class TimerFunction(object):
    def __init__(self, redis):
        self.r = redis

    def register(self, user_id):
        try:
            device_id = "timer-" + user_id
            if self.r.exists("device:" + device_id + ":name") == 0:
                key_pattern = "device:" + device_id
                self.r.set(key_pattern + ":name", "timer")
                self.r.set(key_pattern + ":user_id", user_id)
                return "true"
            else:
                return "false"
        except Exception as error:
            print(repr(error))
            return "error"

    def get_device(self, user_id, device_id):
        try:
            key_pattern = "device:" + device_id
            dto = Timer()
            dto.id = device_id
            dto.name = self.r.get(key_pattern + ":name")
            if self.r.exists(key_pattern + ":rules") == 1:
                rules_id = self.r.lrange(key_pattern + ":rules")
                for rule_id in rules_id:
                    rule_name = self.r.get("user:" + user_id + ":rule:" + rule_id + ":name")
                    dto.rules.append({"id": rule_id, "name": rule_name})
            dto.measure_time = datetime.now().strftime("%H:%M")
            dto.measure_day = str(datetime.today().weekday())
            return dto
        except Exception as error:
            print(repr(error))
            return "error"

    def update_device(self, new_device):
        try:
            dto = Timer()
            dto.device_mapping(new_device)
            key_pattern = "device:" + dto.id
            self.r.set(key_pattern + ":name", dto.name)
            return dto
        except Exception as error:
            print(repr(error))
            return "error"



import jwt


class ProfileFunction(object):
    def __init__(self, redis, token_key):
        self.r = redis
        self.token_key = token_key

    def register(self, dto):
        try:
            output = "false"
            key_pattern = "profile:" + dto.email
            if self.r.exists(key_pattern + ":name") == 0:
                user_id = str(self.r.incr("profile:counter"))
                self.r.set(key_pattern + ":name", dto.name)
                self.r.set(key_pattern + ":password", dto.password)
                self.r.set(key_pattern + ":surname", dto.surname)
                self.r.set(key_pattern + ":user_id", user_id)
                dto.user_id = user_id
                output = self.create_token(dto)
            return output
        except Exception as error:
            print(repr(error))
            return "error"

    def login(self, dto):
        try:
            key_pattern = "profile:" + dto.email
            output = "false"
            if self.r.exists(key_pattern + ":name") == 1:
                password = self.r.get(key_pattern+":password")
                if password == dto.password:
                    dto.name = self.r.get(key_pattern + ":name")
                    dto.surname = self.r.get(key_pattern + ":surname")
                    dto.user_id = self.r.get(key_pattern + ":user_id")
                    output = self.create_token(dto)
            return output
        except Exception as error:
            print(repr(error))
            return "error"

    def create_token(self, dto):
        payload = dto.__dict__
        return jwt.encode(payload, self.token_key, algorithm="HS256")



class UserFunction(object):
    def __init__(self, redis):
        self.r = redis

    def get_sensors(self, user_id):
        try:
            return self.r.lrange("user:" + user_id + ":sensors")
        except Exception as error:
            print(repr(error))
            return "error"

    def get_switches(self, user_id):
        try:
            return self.r.lrange("user:" + user_id + ":switches")
        except Exception as error:
            print(repr(error))
            return "error"

    def get_rules(self, user_id):
        try:
            return self.r.lrange("user:" + user_id + ":rules")
        except Exception as error:
            print(repr(error))
            return "error"

    def get_folders(self, user_id):
        try:
            return self.r.lrange("user:" + user_id + ":folders")
        except Exception as error:
            print(repr(error))
            return "error"





from Devices.Weather.WeatherAntecedentComponents.CloudAntecedentFunctions import CloudAntecedentFunction
from Devices.Weather.WeatherAntecedentComponents.HumidityAntecedentFunctions import HumidityAntecedentFunction
from Devices.Weather.WeatherAntecedentComponents.IconAntecedentFunctions import IconAntecedentFunction
from Devices.Weather.WeatherAntecedentComponents.PressureAntecedentFunctions import PressureAntecedentFunction
from Devices.Weather.WeatherAntecedentComponents.TempAntecedentFunctions import TempAntecedentFunction
from Devices.Weather.WeatherAntecedentComponents.TimeAntecedentFunctions import TimeAntecedentFunction
from Devices.Weather.WeatherAntecedentComponents.WindSpeedAntecedentFunctions import WindSpeedAntecedentFunction
from Devices.Weather.WeatherAntecedentDTO import WeatherAntecedent


class WeatherAntecedentFunction(object):
    def __init__(self, redis):
        self.r = redis
        self.icon_antecedent_functions = IconAntecedentFunction(redis)
        self.temp_antecedent_functions = TempAntecedentFunction(redis)
        self.pressure_antecedent_functions = PressureAntecedentFunction(redis)
        self.humidity_antecedent_functions = HumidityAntecedentFunction(redis)
        self.cloud_antecedent_functions = CloudAntecedentFunction(redis)
        self.time_antecedent_functions = TimeAntecedentFunction(redis)
        self.wind_speed_antecedent_functions = WindSpeedAntecedentFunction(redis)

    def get_antecedent(self, user_id, rule_id, device_id):
        try:
            antecedent = WeatherAntecedent()
            antecedent.device_id = device_id
            antecedent.device_name = self.r.get("device:" + device_id + ":name")
            key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
            antecedent.evaluation = self.r.get(key_pattern + ":evaluation")
            antecedent.set_icon_antecedent(self.icon_antecedent_functions.get_icon_antecedent(user_id, rule_id, device_id))
            antecedent.set_temp_antecedent(self.temp_antecedent_functions.get_temp_antecedent(user_id, rule_id, device_id))
            antecedent.set_pressure_antecedent(
                self.pressure_antecedent_functions.get_pressure_antecedent(user_id, rule_id, device_id))
            antecedent.set_cloud_antecedent(
                self.cloud_antecedent_functions.get_cloud_antecedent(user_id, rule_id, device_id))
            antecedent.set_humidity_antecedent(
                self.humidity_antecedent_functions.get_humidity_antecedent(user_id, rule_id, device_id))
            antecedent.set_time_antecedent(self.time_antecedent_functions.get_time_antecedent(user_id, rule_id, device_id))
            antecedent.set_wind_antecedent(
                self.wind_speed_antecedent_functions.get_wind_antecedent(user_id, rule_id, device_id))
            return antecedent
        except Exception as error:
            print(repr(error))
            return "error"

    def get_antecedent_slim(self, user_id, rule_id, device_id):
        try:
            antecedent = WeatherAntecedent()
            antecedent.device_id = device_id
            antecedent.device_name = self.r.get("device:" + device_id + ":name")
            key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
            antecedent.evaluation = self.r.get(key_pattern + ":evaluation")
            return antecedent
        except Exception as error:
            print(repr(error))
            return "error"

    def set_antecedent(self, user_id, rule_id, device_id, antecedent: WeatherAntecedent):
        try:
            self.icon_antecedent_functions.set_icon_antecedent(user_id, rule_id, device_id,
                                                               antecedent.get_icon_antecedent())
            self.cloud_antecedent_functions.set_cloud_antecedent(user_id, rule_id, device_id,
                                                                 antecedent.get_cloud_antecedent())
            self.humidity_antecedent_functions.set_humidity_antecedent(user_id, rule_id, device_id,
                                                                       antecedent.get_humidity_antecedent())
            self.pressure_antecedent_functions.set_pressure_antecedent(user_id, rule_id, device_id,
                                                                       antecedent.get_pressure_antecedent())
            self.temp_antecedent_functions.set_temp_antecedent(user_id, rule_id, device_id,
                                                               antecedent.get_temp_antecedent())
            self.time_antecedent_functions.set_time_antecedent(user_id, rule_id, device_id,
                                                               antecedent.get_time_antecedent())
            self.wind_speed_antecedent_functions.set_wind_antecedent(user_id, rule_id, device_id,
                                                                     antecedent.get_wind_antecedent())
        except Exception as error:
            print(repr(error))
            return "error"

    def delete_antecedent(self, user_id, rule_id, device_id):
        try:
            self.cloud_antecedent_functions.delete_cloud_antecedent(user_id, rule_id, device_id)
            self.humidity_antecedent_functions.delete_humidity_antecedent(user_id, rule_id, device_id)
            self.icon_antecedent_functions.delete_icon_antecedent(user_id, rule_id, device_id)
            self.pressure_antecedent_functions.delete_pressure_antecedent(user_id, rule_id, device_id)
            self.temp_antecedent_functions.delete_temp_antecedent(user_id, rule_id, device_id)
            self.time_antecedent_functions.delete_time_antecedent(user_id, rule_id, device_id)
            self.wind_speed_antecedent_functions.delete_wind_antecedent(user_id, rule_id, device_id)
            key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
            self.r.delete(key_pattern + ":evaluation")
            self.r.lrem("device:" + device_id + ":rules", rule_id)
            self.r.lrem("user:" + user_id + ":rule:" + rule_id + ":device_antecedents", device_id)
        except Exception as error:
            print(repr(error))
            return "error"

    def register_antecedent(self, user_id, rule_id, device_id):
        try:
            device_antecedents = self.r.lrange("user:" + user_id + ":rule:" + rule_id + ":device_antecedents")
            result = "false"
            if device_id not in device_antecedents:
                self.r.rpush("device:" + device_id + ":rules", rule_id)
                self.r.rpush("user:" + user_id + ":rule:" + rule_id + ":device_antecedents", device_id)
                self.cloud_antecedent_functions.register_cloud_antecedent(user_id, rule_id, device_id)
                self.humidity_antecedent_functions.register_humidity_antecedent(user_id, rule_id, device_id)
                self.icon_antecedent_functions.register_icon_antecedent(user_id, rule_id, device_id)
                self.pressure_antecedent_functions.register_pressure_antecedent(user_id, rule_id, device_id)
                self.temp_antecedent_functions.register_temp_antecedent(user_id, rule_id, device_id)
                self.time_antecedent_functions.register_time_antecedent(user_id, rule_id, device_id)
                self.wind_speed_antecedent_functions.register_wind_speed_antecedent(user_id, rule_id, device_id)
                result = "true"
            return result
        except Exception as error:
            print(repr(error))
            return "error"

    def evalutate_antecedent(self, user_id, rule_id, device_id):
        try:
            key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
            old_evaluation = self.r.get(key_pattern + ":evaluation")
            evaluations = []
            cloud_evaluation = self.cloud_antecedent_functions.clouds_antecedent_evaluation(user_id, rule_id, device_id)
            humidity_evaluation = self.humidity_antecedent_functions.humidity_antecedent_evaluation(user_id, rule_id,
                                                                                                    device_id)
            icon_evaluation = self.icon_antecedent_functions.icon_antecedent_evaluation(user_id, rule_id, device_id)
            pressure_evaluation = self.pressure_antecedent_functions.pressure_antecedent_evaluation(user_id, rule_id,
                                                                                                    device_id)
            temp_evaluation = self.temp_antecedent_functions.temp_antecedent_evaluation(user_id, rule_id, device_id)
            time_evaluation = self.time_antecedent_functions.time_antecedent_evaluation(user_id, rule_id, device_id)
            wind_speed_evaluation = self.wind_speed_antecedent_functions.wind_antecedent_evaluation(user_id, rule_id,
                                                                                                    device_id)
            evaluations.append(cloud_evaluation)
            evaluations.append(humidity_evaluation)
            evaluations.append(icon_evaluation)
            evaluations.append(pressure_evaluation)
            evaluations.append(temp_evaluation)
            evaluations.append(time_evaluation)
            evaluations.append(wind_speed_evaluation)
            evaluation = "true"
            if "false" in evaluations:
                evaluation = "false"
            if old_evaluation != evaluation:
                key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
                self.r.set(key_pattern + ":evaluation", evaluation)
            return evaluation
        except Exception as error:
            print(repr(error))
            return "false"



import redis


class RedisConnection(object):
    def __init__(self, config):
        host = config.get("REDIS", "host")
        port = config.get("REDIS", "port")
        self.r = redis.Redis(host=host, port=port, decode_responses=True)

    def set(self, key, value):
        self.r.set(key, value)

    def get(self, key):
        return self.r.get(key)

    def exists(self, key):
        return self.r.exists(key)

    def setex(self, key, expiration, value):
        self.r.setex(key, expiration, value)

    def smembers(self, key):
        return self.r.smembers(key)

    def lrange(self, key):
        return self.r.lrange(key, 0, -1)

    def scan(self, key):
        return self.r.scan(0, key, 1000)[1]

    def sadd(self, key, value):
        self.r.sadd(key, value)

    def srem(self, key, value):
        self.r.srem(key, value)

    def delete(self, key):
        self.r.delete(key)

    def rpush(self, key, value):
        self.r.rpush(key, value)

    def lrem(self, key, value):
        self.r.lrem(key, 1, value)

    def lset(self, key, idx, value):
        self.r.lset(key, idx, value)

    def incr(self, key):
        return self.r.incr(key)

    def lpush(self, key, value):
        self.r.lpush(key, value)



from Devices.Weather.WeatherAntecedentDTO import TimeAntecedent
from datetime import datetime


class TimeAntecedentFunction(object):
    def __init__(self, redis):
        self.r = redis

    def get_time_antecedent(self, user_id, rule_id, device_id):
        dto = TimeAntecedent()
        key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
        dto.check_time = self.r.get(key_pattern + ":check_time")
        dto.time_condition = self.r.get(key_pattern + ":time_condition")
        dto.time_start_value = self.r.get(key_pattern + ":time_start_value")
        dto.time_stop_value = self.r.get(key_pattern + ":time_stop_value")
        dto.time_evaluation = self.r.get(key_pattern + ":time_evaluation")
        location_name = self.r.get("user:" + user_id + ":location:name")
        dto.sunrise = self.r.get("weather:" + location_name + ":sunrise")
        dto.sunset = self.r.get("weather:" + location_name + ":sunset")
        dto.time_value = datetime.now().strftime("%H:%M")
        return dto

    def set_time_antecedent(self, user_id, rule_id, device_id, dto: TimeAntecedent):
        key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
        self.r.set(key_pattern + ":check_time", dto.check_time)
        self.r.set(key_pattern + ":time_condition", dto.time_condition)
        self.r.set(key_pattern + ":time_start_value", dto.time_start_value)
        self.r.set(key_pattern + ":time_stop_value", dto.time_stop_value)
        self.r.set(key_pattern + ":time_evaluation", dto.time_evaluation)

    def delete_time_antecedent(self, user_id, rule_id, device_id):
        key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
        self.r.delete(key_pattern + ":check_time")
        self.r.delete(key_pattern + ":time_condition")
        self.r.delete(key_pattern + ":time_start_value")
        self.r.delete(key_pattern + ":time_stop_value")
        self.r.delete(key_pattern + ":time_evaluation")

    def register_time_antecedent(self, user_id, rule_id, device_id):
        dto = TimeAntecedent()
        key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
        self.r.set(key_pattern + ":check_time", dto.check_time)
        self.r.set(key_pattern + ":time_condition", dto.time_condition)
        self.r.set(key_pattern + ":time_start_value", dto.time_start_value)
        self.r.set(key_pattern + ":time_stop_value", dto.time_stop_value)
        self.r.set(key_pattern + ":time_evaluation", dto.time_evaluation)

    def time_antecedent_evaluation(self, user_id, rule_id, device_id):
        try:
            dto = self.get_time_antecedent(user_id, rule_id, device_id)
            evaluation = "true"
            if dto.check_time == "true":
                evaluation = "false"
                time_start_value = datetime.strptime(dto.time_start_value, '%H:%M').time()
                time_value = datetime.strptime(dto.time_value, '%H:%M').time()
                time_stop_value = datetime.strptime(dto.time_stop_value, '%H:%M').time()
                if time_start_value < time_value <= time_stop_value:
                    evaluation = "true"
                if dto.time_evaluation != evaluation:
                    key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
                    self.r.set(key_pattern + ":time_evaluation", evaluation)
            return evaluation
        except Exception as error:
            print(repr(error))
            return "false"
